connect sh/sh;
set lines 10000;
set pages 10000;
set long 20000;
set serveroutput on;
SET TRIMSPOOL ON;
SET TAB OFF;
SET FEEDBACK 1;

alter system flush shared_pool; 

set echo on;

-- At this point your previous session has crashed due to a problem 
-- in the compilation. We now want to use the SQL Repair Advisor
-- to fit that problem. 
-- Before we can run the SQL Repair Advisor we need to create a 
-- diagnosis task using dbms_sqldiag. We need to specify the SQL text 
-- for the statement that causes the crash and a task name. We also need 
-- to specify the problem type. There are three possible problem types

-- PROBLEM_TYPE_COMPILATION_ERROR 
-- PROBLEM_TYPE_EXECUTION_ERROR 
--  WRONG_RESULTS
 
-- In this cause since the statement failed during execution we will use 
-- PROBLEM_TYPE_COMPILATION_ERROR. When the SQL Repair Advisor completes 
-- we will run
-- dbms_sqldiag.report_diagnosis_task to output the results report to the screen.

pause 

exec dbms_sqldiag.drop_diagnosis_task('sqldiag_set_to_join');

variable t_id varchar2(50);
variable rep_out clob;

begin
  :t_id := dbms_sqldiag.create_diagnosis_task( 
    sql_text => 'select * from (
select prod_id, cust_id, time_id, channel_id from sales
intersect
select prod_id, cust_id, time_id, channel_id from sales
where channel_id = ''S''
minus
select prod_id, cust_id, time_id, channel_id from sales
where prod_id < 100
order by 1 desc, 2, 3 desc, 4 desc)',
    task_name => 'sqldiag_set_to_join',
    problem_type =>DBMS_SQLDIAG.PROBLEM_TYPE_EXECUTION_ERROR);

  dbms_sqltune.set_tuning_task_parameter(
    :t_id, 
    '_SQLDIAG_FINDING_MODE',
    dbms_sqldiag.SQLDIAG_FINDINGS_FILTER_PLANS);

  dbms_output.put_line ('t_id = ' ||  :t_id);
end;
/

alter session set "_convert_set_to_join"=TRUE;
alter session set "_sdiag_crash"="SET_TO_JOIN";

begin
  dbms_sqldiag.execute_diagnosis_task (:t_id);

  dbms_output.put_line ('executed t_id = ' ||  :t_id);
end;
/

alter session set "_sdiag_crash"="NONE";

begin
  :rep_out := dbms_sqldiag.report_diagnosis_task (:t_id, DBMS_SQLDIAG.TYPE_TEXT);

  dbms_output.put_line ('Report : ' ||  :rep_out);

end;
/

-- The above report tells us that the statement fails during compilation
-- and it has found a SQL patch for this statement that will prevent 
-- it from crashing in the future. However, running the repair advisor 
-- wont fix the problem automatically. If we run our problematic SQL 
-- statement now the session will crash.

pause

connect sh/sh

-- We need to accept the SQL Patch that the SQL Repair Advisor recommended before
-- our SQL statement will be able to run without crashing. To accept the  SQL Patch
-- we must execute dbms_sqldiag.accept_sql_patch and pass it task name

pause
execute dbms_sqldiag.accept_sql_patch(task_name => 'sqldiag_set_to_join', name => 'set_to_join_patch', task_owner => 'SH', replace => TRUE);

-- Now that we have successfully accepted the SQL patch we should be able to 
-- run our statement without it crashing. Lets first look at the execution 
-- plan to see if it has changed.

pause

explain plan for
select * from (
select prod_id, cust_id, time_id, channel_id from sales
intersect
select prod_id, cust_id, time_id, channel_id from sales
where channel_id = 'S'
minus
select prod_id, cust_id, time_id, channel_id from sales
where prod_id < 100
order by 1 desc, 2, 3 desc, 4 desc);

-- display the plan
select plan_table_output from table(dbms_xplan.display('plan_table',null,'advanced'));

 
-- Finally we need to restore the system back the way we found it.
-- We must drop the SQL Patch we have accepted and turn back on the
-- original code path that prevented this problem in the first place.
pause

execute dbms_sqldiag.drop_sql_patch('set_to_join_patch');
