
set echo on


connect scott/tiger
set lines 10000;
set pages 10000;
set long 20000;
set serveroutput on;
SET TRIMSPOOL ON;
SET TAB OFF;
SET FEEDBACK 1;

alter system flush shared_pool; 

set echo on;

-- At this point your previous session has crashed due to a problem 
-- with our delete statement. We now want to use the SQL Repair Advisor
-- to fit that problem. Before we begin to fix this problem lets ensure 
-- that the actual code that prevents this issue in real life is switched off.

pause

alter system set "_fix_control"="5868490:OFF";

-- Before we can run the SQL Repair Advisor we need to create a 
-- diagnosis task using dbms_sqldiag. We need to specify the SQL text 
-- for the statement that causes the crash and a task name. We also need 
-- to specify the problem type. There are three possible problem types

-- PROBLEM_TYPE_COMPILATION_ERROR 
-- PROBLEM_TYPE_EXECUTION_ERROR 
--  WRONG_RESULTS
 
-- In this cause since the statement failed during execution we will use 
-- PROBLEM_TYPE_EXECUTION_ERROR. When the SQL Repair Advisor completes we will run
-- dbms_sqldiag.report_diagnosis_task to output the results report to the screen.

pause 

exec dbms_sqldiag.drop_diagnosis_task('sqldiag_bug_5869490');

declare
  rep_out         clob;
  t_id            varchar2(50);
begin
  t_id := dbms_sqldiag.create_diagnosis_task( 
    sql_text => 'delete   /*+
      USE_HASH_AGGREGATION(@"SEL$80F8B8C6")
      USE_HASH(@"SEL$80F8B8C6" "T1"@"DEL$1")
      LEADING(@"SEL$80F8B8C6" "T2"@"SEL$1" "T1"@"DEL$1")
      FULL(@"SEL$80F8B8C6" "T1"@"DEL$1")
      FULL(@"SEL$80F8B8C6" "T2"@"SEL$1")
      OUTLINE(@"DEL$1")
      OUTLINE(@"SEL$1")
      OUTLINE(@"SEL$AD0B6B07")
      OUTLINE(@"SEL$7D4DB4AA")
      UNNEST(@"SEL$1")
      OUTLINE(@"SEL$75B5BFA2")
      MERGE(@"SEL$7D4DB4AA")
      OUTLINE_LEAF(@"SEL$80F8B8C6")
      ALL_ROWS
      OPT_PARAM(''_optimizer_cost_model'' ''fixed'')
      DB_VERSION(''11.1.0.7'')
      OPTIMIZER_FEATURES_ENABLE(''11.1.0.7'')
      NO_INDEX(@"SEL$1" "T2"@"SEL$1")
  */
from simple_table t1 where t1.a = ''a'' and rowid <> (select max(rowid) from simple_table t2 where t1.a= t2.a and t1.b = t2.b and t1.d=t2.d)',
    task_name => 'sqldiag_bug_5869490',
    problem_type =>DBMS_SQLDIAG.PROBLEM_TYPE_COMPILATION_ERROR);

  dbms_sqltune.set_tuning_task_parameter(
    t_id, 
    '_SQLDIAG_FINDING_MODE',
    dbms_sqldiag.SQLDIAG_FINDINGS_FILTER_PLANS);

  dbms_output.put_line ('t_id = ' ||  t_id);

  dbms_sqldiag.execute_diagnosis_task (t_id);

  dbms_output.put_line ('executed t_id = ' ||  t_id);

  rep_out := dbms_sqldiag.report_diagnosis_task (t_id, DBMS_SQLDIAG.TYPE_TEXT);

  dbms_output.put_line ('Report : ' ||  rep_out);

end;
/

-- The above report tells us that the statement fails during execute 
-- and it has found a SQL patch for this statement that will prevent 
-- it from crashing in the future. However, unning the repair advisor 
-- wont fix the problem automatically. If we run our problematic SQL 
-- statement now the session will crash.

pause

connect scott/tiger

-- We need to accept the SQL Patch that the SQL Repair Advisor recommended before
-- our SQL statement will be able to run without crashing. To accept the  SQL Patch
-- we must execute dbms_sqldiag.accept_sql_patch and pass it task name

pause
execute dbms_sqldiag.accept_sql_patch(task_name => 'sqldiag_bug_5869490', name => 'delete_patch', task_owner => 'SCOTT', replace => TRUE);

-- Now that we have successfully accepted the SQL patch we should be able to 
-- run our statement without it crashing. Lets first look at the execution 
-- plan to see if it has changed.

pause

explain plan for
delete 
  /*+
      USE_HASH_AGGREGATION(@"SEL$80F8B8C6")
      USE_HASH(@"SEL$80F8B8C6" "T1"@"DEL$1")
      LEADING(@"SEL$80F8B8C6" "T2"@"SEL$1" "T1"@"DEL$1")
      FULL(@"SEL$80F8B8C6" "T1"@"DEL$1")
      FULL(@"SEL$80F8B8C6" "T2"@"SEL$1")
      OUTLINE(@"DEL$1")
      OUTLINE(@"SEL$1")
      OUTLINE(@"SEL$AD0B6B07")
      OUTLINE(@"SEL$7D4DB4AA")
      UNNEST(@"SEL$1")
      OUTLINE(@"SEL$75B5BFA2")
      MERGE(@"SEL$7D4DB4AA")
      OUTLINE_LEAF(@"SEL$80F8B8C6")
      ALL_ROWS
      OPT_PARAM('_optimizer_cost_model' 'fixed')
      DB_VERSION('11.1.0.7')
      OPTIMIZER_FEATURES_ENABLE('11.1.0.7')
      NO_INDEX(@"SEL$1" "T2"@"SEL$1")
  */
from simple_table t1 where t1.a = 'a' and rowid <> (select max(rowid) from simple_table t2 where t1.a= t2.a and t1.b = t2.b and t1.d=t2.d);

-- display the plan
select plan_table_output from table(dbms_xplan.display('plan_table',null,'advanced'));

-- Now lets execute it.

pause

delete 
  /*+
      USE_HASH_AGGREGATION(@"SEL$80F8B8C6")
      USE_HASH(@"SEL$80F8B8C6" "T1"@"DEL$1")
      LEADING(@"SEL$80F8B8C6" "T2"@"SEL$1" "T1"@"DEL$1")
      FULL(@"SEL$80F8B8C6" "T1"@"DEL$1")
      FULL(@"SEL$80F8B8C6" "T2"@"SEL$1")
      OUTLINE(@"DEL$1")
      OUTLINE(@"SEL$1")
      OUTLINE(@"SEL$AD0B6B07")
      OUTLINE(@"SEL$7D4DB4AA")
      UNNEST(@"SEL$1")
      OUTLINE(@"SEL$75B5BFA2")
      MERGE(@"SEL$7D4DB4AA")
      OUTLINE_LEAF(@"SEL$80F8B8C6")
      ALL_ROWS
      OPT_PARAM('_optimizer_cost_model' 'fixed')
      DB_VERSION('11.1.0.7')
      OPTIMIZER_FEATURES_ENABLE('11.1.0.7')
      NO_INDEX(@"SEL$1" "T2"@"SEL$1")
  */
from simple_table t1 where t1.a = 'a' and rowid <> (select max(rowid) from simple_table t2 where t1.a= t2.a and t1.b = t2.b and t1.d=t2.d);
 
-- Finally we need to restore the system back the way we found it.
-- We must drop the SQL Patch we have accepted and turn back on the
-- original code path that prevented this problem in the first place.

execute dbms_sqldiag.drop_sql_patch('delete_patch');

alter system set "_fix_control"="5868490:ON";
